/*
 * GWT - General Windowing Toolkit
 *
 * Copyright (C) 1998 MenTaLguY - mentalg@geocities.com
 *                    Rodolphe Ortalo - ortalo@laas.fr
 *                    Tristan Wibberley - bloater@ps.cus.umist.ac.uk
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: invalidate.c,v $
 * Revision 1.10  2000/03/19 19:32:46  ortalo
 * Some debugging and modifications (wrt OpenAmulet mainly).
 *
 * Revision 1.9  2000/03/13 22:02:55  ortalo
 * Committed new version of LibGWT corresponding to 0.0.2.
 * The event management API is probably more or less defined. The simplest
 * approach finally won. (I should have remembered: KISS!)
 *
 * Revision 1.8  1999/05/20 20:34:23  ortalo
 * Several bugs corrections and small improvements.
 *
 * Revision 1.7  1999/05/13 20:01:38  ortalo
 * New version of libgwt. Now using LibGII for input management.
 * Some drawing functions implemented. Visibility management should
 * be rather okay. Still some work to do, but now, your eyes can
 * look at some drawings... not only code. ;-)
 *
 * Revision 1.6  1998/11/17 15:13:28  tristan
 *
 * Changed my email address
 *
 * Revision 1.5  1998/09/02 02:21:46  tristan
 * Loadsa great stuff done. Visible/invalid regions,
 *  copy on move - fixed tt_region.c too!
 *
 * Revision 1.4  1998/08/23 18:57:12  ortalo
 * Minor editing (mainly comments).
 *
 * Revision 1.3  1998/08/16 18:40:56  ortalo
 * A new version, STILL IN VERY ALPHA STATE... (Be patient.)
 * (However, this version is a little better... :-)
 *
 * Revision 1.2  1998/08/11 22:10:09  ortalo
 * Incorporated the changes made by Tristan Wibberley. The code is
 * still in alpha development state.
 *
 * Revision 1.1  1998/07/09 18:38:39  ortalo
 * Initial integration of libgwt in the GGI repository.
 * This is experimental code, for your eyes only !
 * (Don't execute... ;-)
 *
 */

#include <stdlib.h>
#include <stdio.h>

#include "internal.h"
#include "gwt_debug.h"

/*******************************************************************
 *                  GWT PRIVATE EXPORTED FUNCTIONS                 *
 *******************************************************************/

int gwt_invalidate(gwt_window_t win, gwt_region_t region)
{
  gwt_region_t newr, exposed;
  gwt_region_t tmp;
  /* Copy the region given */
  gwt_region_create(&newr);
  gwt_region_copy(newr, region);
  /* Unify with previous invalid region */
  gwt_region_union(newr, win->invalid);
  /* Intersect it with the visible region */
  /* (We could skip this step to have a better speed ?) */
  gwt_region_intersection(newr, win->visible);
  /* Compute the exposed region*/
  gwt_region_create(&exposed);
  gwt_region_copy(exposed, newr);
  gwt_region_difference(exposed, win->invalid);
  tmp = win->invalid;      /* Remember previous one... */
  /* Update the invalid region */
  win->invalid = newr;
  gwt_region_destroy(tmp); /* ...and destroy it */
  { /* Send an exposure event */
    ggi_coord tl, br;
    gii_event ev;
    ev.any.size = sizeof(gii_expose_event);
    ev.any.type = evExpose;
    ev.any.target = GII_EV_TARGET_QUEUE;
    ev.any.origin = GII_EV_ORIGIN_SENDEVENT; /* Needed ? */
    gwt_region_bounds(exposed, &tl, &br);
    ev.expose.x = tl.x - win->origin.x; /* Sends the event in window coordinates */
    ev.expose.y = tl.y - win->origin.y;
    ev.expose.w = br.x - tl.x;
    ev.expose.h = br.y - tl.y;
    if ( (ev.expose.w != 0) || (ev.expose.h != 0) )
      {
	int ret = gwtEventSend(win, &ev);
	LIBGWT_ASSERT(ret == GGI_OK, "Error in sending invalidate event!");
      }
  }
  /* Destroy last things */
  gwt_region_destroy(exposed);
  /* Now pass the information to children... */
  {
    gwt_window_t child = GWT_Z_BOTTOM_CHILD(win);
    while (child != NULL)
      {
	gwt_invalidate(child, region);
	child = GWT_Z_UP(child);
      }
  }
  return 0;
}

int gwt_validate(gwt_window_t win, gwt_region_t region)
{
  return gwt_region_difference(win->invalid, region);
}

int gwt_invalidate_all(gwt_window_t win)
{
  return gwt_invalidate(win, win->geometry);
}

int gwt_validate_all(gwt_window_t win)
{
  return gwt_validate(win, win->geometry);
}

/*******************************************************************
 *                  GWT PUBLIC EXPORTED FUNCTIONS                  *
 *******************************************************************/

int gwtInvalidate(gwt_window_t win, gwt_region_t region)
{
  gwt_region_t tmp;
  int err;
  gwt_region_create(&tmp);
  gwt_region_copy(tmp, region);
  gwt_region_move(tmp, win->origin.x, win->origin.y);
  err = gwt_invalidate(win, tmp);
  gwt_region_destroy(tmp);
  return err;
}
int gwtValidate(gwt_window_t win, gwt_region_t region)
{
  gwt_region_t tmp;
  int err;
  gwt_region_create(&tmp);
  gwt_region_copy(tmp, region);
  gwt_region_move(tmp, win->origin.x, win->origin.y);
  err = gwt_validate(win, tmp);
  gwt_region_destroy(tmp);
  return err;
}

int gwtValidateAll(gwt_window_t win)
{
  return gwt_validate_all(win);
}
int gwtInvalidateAll(gwt_window_t win)
{
  return gwt_invalidate_all(win);
}
